import time

from can_id_sheet import VehicleSegment, CANFrame
from ecu_process import GenericECU

from scapy.config import conf
# from scapy.contrib.cansocket_python_can import CANSocket
from scapy.contrib.cansocket import CANSocket
from scapy.main import load_contrib
from time import sleep
import multiprocessing as mp
import can


def coordinate_python_can():
    # create a bus instance using 'with' statement,
    # this will cause bus.shutdown() to be called on the block exit;
    # many other interfaces are supported as well (see documentation)
    with can.Bus(interface='socketcan', channel='vcan0', receive_own_messages=True) as bus:
        # send a message
        message = can.Message(arbitration_id=123, is_extended_id=False,
                              data=[0x11, 0x22, 0x33])
        bus.send(message, timeout=0.2)

        # iterate over received messages
        # for msg in bus:
        #     print(f"{msg.arbitration_id:X}: {msg.data}")

        # or use an asynchronous notifier
        _ = can.Notifier(bus, [can.Logger("recorded.log"), can.Printer()])
        sleep(5)


def get_filtered_can_socket(vehicle_segment: VehicleSegment):
    filters = [{'can_id': f.value, 'can_mask': 0x7FF} for f in CANFrame if
               f.execution == vehicle_segment]
    # return CANSocket(bustype='socketcan', channel='vcan0', bitrate=250000, can_filters=filters)
    return CANSocket(channel='vcan0', can_filters=filters)


if __name__ == '__main__':
    # CAN related setup
    # load_layer("can")
    # load_contrib('cansocket')

    conf.contribs['CANSocket'] = {'use-python-can': True}
    load_contrib('cansocket')
    # socket = CANSocket(bustype='virtual', channel='vcan0', bitrate=250000)
    socket = CANSocket(channel='vcan0')

    # multiprocessing setup
    mp.set_start_method('fork')
    processes = [
        GenericECU(tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.Chassis),
                   vehicle_segment=VehicleSegment.Chassis),
        GenericECU(tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.PowerTrain),
                   vehicle_segment=VehicleSegment.PowerTrain),
        GenericECU(tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.Body),
                   vehicle_segment=VehicleSegment.Body)
    ]

    for p in processes:
        p.start()

    for p in processes:
        p.join()
