from can_id_sheet import CANFrame

from threading import Thread
from scapy.contrib.cansocket import *
from scapy.layers.can import *
from scapy.all import *
# from scapy.contrib.cansocket_python_can import CANSocket
from scapy.sendrecv import send
import time

NANOSECONDS_IN_SECOND: int = 1_000_000_000


class PeriodicSendingTask(Thread):
    def __init__(self,
                 socket,  # type: CANSocket,
                 frames: list[CANFrame],
                 interval_s: float):
        super().__init__()
        self.daemon = True
        self.stopped = False    # maybe add a way to gracefully stop
        self.socket = socket
        self.frames = frames
        self.interval_s = interval_s
        self.interval_ns = int(round(interval_s * NANOSECONDS_IN_SECOND))
        self.packets = [CAN(identifier=f.value, data=b'01020304') for f in frames]

    def run(self) -> None:
        msg_due_time_ns = time.perf_counter_ns()

        while not self.stopped:
            send(self.packets, verbose=0, socket=self.socket)
            print(f'Sent {len(self.packets)} packets at {time.time()}, frequency = {self.interval_s}s')
            # TODO: implement ethernet packet sending
            # netpacket = Raw('\x01\x02\x03\x04')
            # sendp(netpacket, iface="lo")

            msg_due_time_ns += self.interval_ns
            # Compensate for the time it takes to send the message
            delay_ns = msg_due_time_ns - time.perf_counter_ns()
            if delay_ns > 0:
                time.sleep(delay_ns / NANOSECONDS_IN_SECOND)
